@@ -1475,7 +1475,8 @@ static const sqlite3_module amatchModule = {
   0,                      /* xSavepoint */
   0,                      /* xRelease */
   0,                      /* xRollbackTo */
-  0                       /* xShadowName */
+  0,                      /* xShadowName */
+  0                       /* xIntegrity */
 };
 
 #endif /* SQLITE_OMIT_VIRTUALTABLE */
