@@ -588,12 +588,17 @@ static int closureOpen(sqlite3_vtab *pVTab, sqlite3_vtab_cursor **ppCursor){
   return SQLITE_OK;
 }
 
+/*
+** Wrapper around sqlite3_free
+*/
+static void closureMemFree(closure_avl *p){ sqlite3_free(p); }
+
 /*
 ** Free up all the memory allocated by a cursor.  Set it rLimit to 0
 ** to indicate that it is at EOF.
 */
 static void closureClearCursor(closure_cursor *pCur){
-  closureAvlDestroy(pCur->pClosure, (void(*)(closure_avl*))sqlite3_free);
+  closureAvlDestroy(pCur->pClosure, closureMemFree);
   sqlite3_free(pCur->zTableName);
   sqlite3_free(pCur->zIdColumn);
   sqlite3_free(pCur->zParentColumn);
@@ -939,7 +944,8 @@ static const sqlite3_module closureModule = {
   0,                      /* xSavepoint */
   0,                      /* xRelease */
   0,                      /* xRollbackTo */
-  0                       /* xShadowName */
+  0,                      /* xShadowName */
+  0                       /* xIntegrity */
 };
 
 #endif /* SQLITE_OMIT_VIRTUALTABLE */
