@@ -846,8 +846,7 @@ __declspec(dllexport)
 #endif
 int sqlite3_register_vfslog(const char *zArg){
   vlog_vfs.pVfs = sqlite3_vfs_find(0);
-  if( vlog_vfs.pVfs==&vlog_vfs.base ) return 0; /* Already registered. */
-  vlog_vfs.base.iVersion = vlog_vfs.pVfs->iVersion;
+  if( vlog_vfs.pVfs==0 ) return SQLITE_ERROR;
   vlog_vfs.base.szOsFile = sizeof(VLogFile) + vlog_vfs.pVfs->szOsFile;
   vlog_vfs.base.mxPathname = vlog_vfs.pVfs->mxPathname;
   return sqlite3_vfs_register(&vlog_vfs.base, 1);
